@@ -1,5 +1,6 @@
 package Config::GitLike;
-use Any::Moose;
+use Moo;
+use MooX::Types::MooseLike::Base qw(Bool HashRef ArrayRef Maybe Str);
 
 use File::Spec;
 use Cwd;
@@ -7,13 +8,13 @@ use Scalar::Util qw(openhandle);
 use Fcntl qw(O_CREAT O_EXCL O_WRONLY);
 use 5.008;
 
-our $VERSION = '1.09';
+our $VERSION = '1.11';
 
 
 has 'confname' => (
     is => 'rw',
     required => 1,
-    isa => 'Str',
+    isa => Str,
 );
 
 # not defaulting to {} allows the predicate is_loaded
@@ -21,26 +22,32 @@ has 'confname' => (
 has 'data' => (
     is => 'rw',
     predicate => 'is_loaded',
-    isa => 'HashRef',
+    isa => HashRef,
 );
 
 # key => bool
 has 'multiple' => (
     is => 'rw',
-    isa => 'HashRef',
+    isa => HashRef,
+    default => sub { +{} },
+);
+
+has 'casing' => (
+    is => 'rw',
+    isa => HashRef,
     default => sub { +{} },
 );
 
 # filename where the definition of each key was loaded from
 has 'origins' => (
     is => 'rw',
-    isa => 'HashRef',
+    isa => HashRef,
     default => sub { +{} },
 );
 
 has 'config_files' => (
     is => 'rw',
-    isa => 'ArrayRef',
+    isa => ArrayRef,
     default => sub { [] },
 );
 
@@ -48,32 +55,32 @@ has 'config_files' => (
 # of only-write-things-that-git-config-can-read if you want to
 has 'compatible' => (
     is => 'rw',
-    isa => 'Bool',
-    default => 0,
+    isa => Bool,
+    default => sub { 0 },
 );
 
 has 'cascade' => (
     is => 'rw',
-    isa => 'Bool',
-    default => 0,
+    isa => Bool,
+    default => sub { 0 },
 );
 
 has 'encoding' => (
     is => 'rw',
-    isa => 'Maybe[Str]',
+    isa => Maybe[Str],
 );
 
 sub set_multiple {
     my $self = shift;
     my ($name, $mult) = (@_, 1);
-    $self->multiple->{$name} = $mult;
+    $self->multiple->{ $self->canonical_case( $name ) } = $mult;
 }
 
 sub is_multiple {
     my $self = shift;
     my $name = shift;
     return if !defined $name;
-    return $self->multiple->{$name};
+    return $self->multiple->{ $self->canonical_case( $name ) };
 }
 
 sub load {
@@ -301,7 +308,7 @@ sub parse_content {
             ) unless defined $section;
             $args{callback}->(
                 section    => $section,
-                name       => lc $1,
+                name       => $1,
                 offset     => $offset,
                 length     => ($length - length($c)) - $offset,
             );
@@ -313,7 +320,7 @@ sub parse_content {
                 content => $args{content},
                 offset  => $offset,
             ) unless defined $section;
-            my $name = lc $1;
+            my $name = $1;
             my $value = "";
             # parse the value
             while (1) {
@@ -413,6 +420,7 @@ sub define {
         @_,
     );
     return unless defined $args{section} and defined $args{name};
+    my $original_key = join(".", @args{qw/section name/});
     $args{name} = lc $args{name};
     my $key = join(".", @args{qw/section name/});
 
@@ -422,13 +430,16 @@ sub define {
         || $self->origins->{$key} eq $args{origin} ) {
         if ($self->is_multiple($key)) {
             push @{$self->data->{$key} ||= []}, $args{value};
+            push @{$self->casing->{$key} ||= []}, $original_key;
         }
         elsif (exists $self->data->{$key}) {
             $self->set_multiple($key);
             $self->data->{$key} = [$self->data->{$key}, $args{value}];
+            $self->casing->{$key}  = [$self->casing->{$key}, $original_key];
         }
         else {
             $self->data->{$key} = $args{value};
+            $self->casing->{$key} = $original_key;
         }
     }
     # we're overriding a key set previously from a different file
@@ -438,6 +449,7 @@ sub define {
 
         # set the new value
         $self->data->{$key} = $args{value};
+        $self->casing->{$key} = $original_key;
     }
     $self->origins->{$key} = $args{origin};
 }
@@ -520,10 +532,7 @@ sub _get {
     );
     $self->load unless $self->is_loaded;
 
-    my ($section, $subsection, $name) = _split_key($args{key});
-    $args{key} = join( '.',
-        grep { defined } (lc $section, $subsection, lc $name),
-    );
+    $args{key} = $self->canonical_case( $args{key} );
 
     return () unless exists $self->data->{$args{key}};
     my $v = $self->data->{$args{key}};
@@ -635,6 +644,21 @@ sub get_regexp {
     return wantarray ? %results : \%results;
 }
 
+sub original_key {
+    my $self = shift;
+    my ($key) = @_;
+    return $self->casing->{ $self->canonical_case( $key ) };
+}
+
+sub canonical_case {
+    my $self = shift;
+    my ($key) = @_;
+    my ($section, $subsection, $name) = _split_key($key);
+    return join( '.',
+        grep { defined } (lc $section, $subsection, lc $name),
+    );
+}
+
 sub dump {
     my $self = shift;
 
@@ -646,6 +670,8 @@ sub dump {
     for my $key (sort keys %{$self->data}) {
         my $str;
         if (defined $self->data->{$key}) {
+            # For git compat, we intentionally always write out in
+            # canonical (i.e. lower) case.
             $str = "$key=";
             if ( $self->is_multiple($key) ) {
                 $str .= '[';
@@ -751,12 +777,6 @@ sub group_set {
         my %args = %{$args_hash};
 
         my ($section, $subsection, $name) = _split_key($args{key});
-        my $key = join( '.',
-            grep { defined } (lc $section, $subsection, lc $name),
-        );
-
-        $args{multiple} = $self->is_multiple($key)
-            unless defined $args{multiple};
 
         die "No section given in key or invalid key $args{key}\n"
             unless defined $section;
@@ -785,6 +805,11 @@ sub group_set {
         my $new;
         my @replace;
 
+        my $key = $self->canonical_case( $args{key} );
+
+        $args{multiple} = $self->is_multiple($key)
+            unless defined $args{multiple};
+
         # use this for comparison
         my $cmp_section =
           defined $unescaped_subsection
@@ -813,7 +838,7 @@ sub group_set {
 
                 my $matched = 0;
                 # variable names are case-insensitive
-                if (lc $name eq $got{name}) {
+                if (lc $name eq lc $got{name}) {
                     if (defined $args{filter} and length $args{filter}) {
                         # copy the filter arg here since this callback may
                         # be called multiple times and we don't want to
@@ -1295,7 +1320,7 @@ written by git. The converse is usually true, but only if you don't take
 advantage of this module's increased permissiveness when it comes to key
 names. (See L<DIFFERENCES FROM GIT-CONFIG> for details.)
 
-This is an object-oriented module using L<Any::Moose|Any::Moose>. All
+This is an object-oriented module using L<Moo|Moo>. All
 subroutines are object method calls.
 
 A few methods have parameters that are always used for the same purpose:
@@ -1753,6 +1778,19 @@ Return a string containing the key/value pair as they should be printed in the
 config file. If C<bare> is true, the returned value is not tab-indented nor
 followed by a newline.
 
+=head2 canonical_case( $name )
+
+Given a full key name, returns the canonical name of the key; this is
+the key with the section and name lower-cased; the subsection is left
+as-is.
+
+=head2 original_key( $name )
+
+Given a full key name, returns the key as it was last loaded from the
+file, retaining what ever upper/lower case was used.  Note that for
+multiple-valued keys, this returns an array reference of key names, as
+each definition may have been provided in a different choice of case.
+
 =head1 DIFFERENCES FROM GIT-CONFIG
 
 This module does the following things differently from git-config:
